<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\Crud; 
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;
use Teknisa\Libs\Util\Environment;

class ConsumidorFamilia extends Crud{
    
    protected $dataSourceName = 'vnd09500_consumidorPorFamilia';
    protected $util;
    protected $ConsumidorFamiliaService;
    protected $environment;
    
    
    public function __construct(Manager $dataSourceManager, Services\ConsumidorFamiliaService $ConsumidorFamiliaService, Utilities\Util $util, Environment $environment) {
        parent::__construct($dataSourceManager);
        $this->ConsumidorFamiliaService = $ConsumidorFamiliaService;
        $this->util                     = $util;
        $this->environment              = $environment;
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response){
        
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $cdValid = $this->ConsumidorFamiliaService->validCodigo($row['CDCLIENTE'], $row['CDFAMILISALD'], $row['CDCONSUMIDOR']);
                if (isset($cdValid[0][1]) && $row['__is_new']){
                    $mensagem =  'Operação não permitida. Código '.$row['CDFAMILISALD'].' já associado.'; 
                    throw new \Exception($mensagem);
                } else {
                    $row['VRSALDCONFAM'] = (float)$row['VRSALDCONFAM'];
                    $row['VRSALDCONFAM_inline'] = $row['VRSALDCONFAM']; 
                }
            }
            parent::save($request,$response);
            $this->saveExtrato($request);
        } catch (\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    public function saveExtrato($request) {
        $rows = $request->getDataSet()->getRows();
        
        $cdtiporece = $this->ConsumidorFamiliaService->findTipoRece()['CDTIPORECE'];
        if(count($rows) > 1) {
            foreach($rows as $saldo) {
                $saldo['CDTIPORECE'] = $cdtiporece;
                $saldo['NRSEQREGCREDVND'] = $this->util->newCode('NRSEQREGCREDVND', '20');
                
                if(isset($saldo['changed_VRSALDCONFAM'])) {
                    $extratoValue = round(($saldo['VRSALDCONFAM_inline'] - (float)$saldo['changed_VRSALDCONFAM']), 2);
                    $teste = abs($extratoValue);
                    $params = $this->prepareParams($saldo, $extratoValue, $teste);
                    $this->ConsumidorFamiliaService->insertExtrato($params);
                }
            }
        } else {
            $saldo = $rows[0];

            $saldo['CDTIPORECE'] = $cdtiporece;
            $saldo['NRSEQREGCREDVND'] = $this->util->newCode('NRSEQREGCREDVND', '20');
            
            if(!$saldo['__is_new']) {
                $extratoValue = round(($saldo['VRSALDCONFAM_inline'] - (float)$saldo['changed_VRSALDCONFAM']), 2);
                $teste = abs($extratoValue);
                $params = $this->prepareParams($saldo, $extratoValue, $teste);
                $this->ConsumidorFamiliaService->insertExtrato($params);
            } else {
                $extratoValue = round(($saldo['VRSALDCONFAM_inline']), 2);
                $teste = abs($extratoValue);
                $params = $this->prepareParams($saldo, $extratoValue, $teste);
                $this->ConsumidorFamiliaService->insertExtrato($params);
            }
        }
        
    }
    
    public function prepareParams($row, $VRMOVEXTCONS, $teste){
        return $params = array(
            ':CDCLIENTE'       => $row['CDCLIENTE'],
            ':CDCONSUMIDOR'    => $row['CDCONSUMIDOR'],
            ':CDFAMILISALD'    => $row['CDFAMILISALD'],
            ':NRSEQMOVEXT'     => '001',
            ':VRMOVEXTCONS'    => $teste,
            ':VRSALDCONEXT'    => $row['VRSALDCONFAM'],
            ':IDTPMOVEXT'      => ($VRMOVEXTCONS < 0 ? 'D' : 'C'),
            ':CDOPERINCLUSAO'  => $this->environment->getCdOperador(),
            ':CDFILIAL'        => $row['CDFILIAL'],
            ':CDTIPORECE'      => $row['CDTIPORECE'],
            ':NRSEQREGCREDVND' => $row['NRSEQREGCREDVND']
        );
    }
}